@@ -7,7 +7,7 @@ from abc import abstractmethod
 from warnings import warn
 
 from aiohttp import ClientSession, ClientResponse
-from typing import Any, Callable, Generator, Optional, Set, Type, Iterable, List
+from typing import Any, Callable, Generator, Optional, Set, Type, Iterable, List, Union
 
 if sys.version_info >= (3, 8):
     from typing import Protocol
@@ -27,6 +27,9 @@ class _Logger(Protocol):
     def warning(self, msg: str, *args: Any, **kwargs: Any) -> None: pass
 
 
+_URL_TYPE = Union[str, List[str]]  # url itself or list of urls for changing between retries
+
+
 class RetryOptionsBase:
     def __init__(
         self,
@@ -115,17 +118,19 @@ class _RequestContext:
     def __init__(
         self,
         request: Callable[..., Any],  # Request operation, like POST or GET
-        url: str,  # Just url
+        urls: List[str],  # Just url
         logger: _Logger,
         retry_options: RetryOptionsBase,
+        raise_for_status: bool = False,
         **kwargs: Any
     ) -> None:
         self._request = request
-        self._url = url
+        self._urls = urls
         self._logger = logger
         self._retry_options = retry_options
         self._kwargs = kwargs
         self._trace_request_ctx = kwargs.pop('trace_request_ctx', {})
+        self._raise_for_status = raise_for_status
 
         self._current_attempt = 0
         self._response: Optional[ClientResponse] = None
@@ -139,7 +144,7 @@ class _RequestContext:
             self._logger.debug("Attempt {} out of {}".format(self._current_attempt, self._retry_options.attempts))
 
             response: ClientResponse = await self._request(
-                self._url,
+                self._urls[self._current_attempt - 1],
                 **self._kwargs,
                 trace_request_ctx={
                     'current_attempt': self._current_attempt,
@@ -153,6 +158,8 @@ class _RequestContext:
                 await asyncio.sleep(retry_wait)
                 return await self._do_request()
             self._response = response
+            if self._raise_for_status:
+                response.raise_for_status()
             return response
 
         except Exception as e:
@@ -182,6 +189,7 @@ class RetryClient:
         self,
         logger: Optional[_Logger] = None,
         retry_options: RetryOptionsBase = ExponentialRetry(),
+        raise_for_status: bool = False,
         *args: Any, **kwargs: Any
     ) -> None:
         self._client = ClientSession(*args, **kwargs)
@@ -192,6 +200,7 @@ class RetryClient:
 
         self._logger: _Logger = logger
         self._retry_options: RetryOptionsBase = retry_options
+        self._raise_for_status = raise_for_status
 
     def __del__(self) -> None:
         if not self._closed:
@@ -200,35 +209,84 @@ class RetryClient:
     def _request(
         self,
         request: Callable[..., Any],
-        url: str,
+        url: _URL_TYPE,
         logger: _Logger,
         retry_options: Optional[RetryOptionsBase] = None,
+        raise_for_status: Optional[bool] = None,
         **kwargs: Any
     ) -> _RequestContext:
         if retry_options is None:
             retry_options = self._retry_options
-        return _RequestContext(request, url, logger, retry_options, **kwargs)
+        if isinstance(url, str):
+            urls = [url] * retry_options.attempts
+        elif not isinstance(url, list) or len(url) == 0:
+            raise ValueError("you can pass url by str or list with attempts count size")
+        elif len(url) < retry_options.attempts:
+            urls = url.copy()
+            last_request_urls = [url[-1]] * (retry_options.attempts - len(url))
+            urls.extend(last_request_urls)
+        else:
+            urls = url
 
-    def get(self, url: str, retry_options: Optional[RetryOptionsBase] = None, **kwargs: Any) -> _RequestContext:
-        return self._request(self._client.get, url, self._logger, retry_options, **kwargs)
+        if raise_for_status is None:
+            raise_for_status = self._raise_for_status
+        return _RequestContext(request, urls, logger, retry_options, raise_for_status, **kwargs)
 
-    def options(self, url: str, retry_options: Optional[RetryOptionsBase] = None, **kwargs: Any) -> _RequestContext:
-        return self._request(self._client.options, url, self._logger, retry_options, **kwargs)
+    def get(
+        self, url: _URL_TYPE,
+        retry_options: Optional[RetryOptionsBase] = None,
+        raise_for_status: Optional[bool] = None,
+        **kwargs: Any
+    ) -> _RequestContext:
+        return self._request(self._client.get, url, self._logger, retry_options, raise_for_status, **kwargs)
 
-    def head(self, url: str, retry_options: Optional[RetryOptionsBase] = None, **kwargs: Any) -> _RequestContext:
-        return self._request(self._client.head, url, self._logger, retry_options, **kwargs)
+    def options(
+        self, url: _URL_TYPE,
+        retry_options: Optional[RetryOptionsBase] = None,
+        raise_for_status: Optional[bool] = None,
+        **kwargs: Any
+    ) -> _RequestContext:
+        return self._request(self._client.options, url, self._logger, retry_options, raise_for_status, **kwargs)
 
-    def post(self, url: str, retry_options: Optional[RetryOptionsBase] = None, **kwargs: Any) -> _RequestContext:
-        return self._request(self._client.post, url, self._logger, retry_options, **kwargs)
+    def head(
+        self, url: _URL_TYPE,
+        retry_options: Optional[RetryOptionsBase] = None,
+        raise_for_status: Optional[bool] = None, **kwargs: Any
+    ) -> _RequestContext:
+        return self._request(self._client.head, url, self._logger, retry_options, raise_for_status, **kwargs)
 
-    def put(self, url: str, retry_options: Optional[RetryOptionsBase] = None, **kwargs: Any) -> _RequestContext:
-        return self._request(self._client.put, url, self._logger, retry_options, **kwargs)
+    def post(
+        self, url: _URL_TYPE,
+        retry_options: Optional[RetryOptionsBase] = None,
+        raise_for_status: Optional[bool] = None,
+        **kwargs: Any
+    ) -> _RequestContext:
+        return self._request(self._client.post, url, self._logger, retry_options, raise_for_status, **kwargs)
 
-    def patch(self, url: str, retry_options: Optional[RetryOptionsBase] = None, **kwargs: Any) -> _RequestContext:
-        return self._request(self._client.patch, url, self._logger, retry_options, **kwargs)
+    def put(
+        self, url: _URL_TYPE,
+        retry_options: Optional[RetryOptionsBase] = None,
+        raise_for_status: Optional[bool] = None,
+        **kwargs: Any
+    ) -> _RequestContext:
+        return self._request(self._client.put, url, self._logger, retry_options, raise_for_status, **kwargs)
 
-    def delete(self, url: str, retry_options: Optional[RetryOptionsBase] = None, **kwargs: Any) -> _RequestContext:
-        return self._request(self._client.delete, url, self._logger, retry_options, **kwargs)
+    def patch(
+        self, url: _URL_TYPE,
+        retry_options: Optional[RetryOptionsBase] = None,
+        raise_for_status: Optional[bool] = None,
+        **kwargs: Any
+    ) -> _RequestContext:
+        return self._request(self._client.patch, url, self._logger, retry_options, raise_for_status, **kwargs)
+
+    def delete(
+        self,
+        url: _URL_TYPE,
+        retry_options: Optional[RetryOptionsBase] = None,
+        raise_for_status: Optional[bool] = None,
+        **kwargs: Any
+    ) -> _RequestContext:
+        return self._request(self._client.delete, url, self._logger, retry_options, raise_for_status, **kwargs)
 
     async def close(self) -> None:
         await self._client.close()
